from pcluster import utils

PCLUSTER_VERSION = utils.get_installed_version()


def check_pcluster_version(pcluster_version: str = "3.2"):
    if pcluster_version not in PCLUSTER_VERSION:
        w = f"""Mismatch between specified pcluster version and installed
        Specified: {pcluster_version}, Installed: {PCLUSTER_VERSION}
        """
        raise ValueError(w)



import os
import boto3

import questionary
from pcluster.cli.commands.configure.easyconfig import (
    _create_vpc_parameters,
    _choose_network_configuration,
    _get_common_supported_az_for_multi_instance_types,
)
from pcluster.cli.commands.configure.easyconfig import (
    _get_vpcs_and_subnets,
)
from aws_pcluster_bootstrap_helpers.utils.constants import (
    MAX_NUMBER_OF_QUEUES,
    MAX_NUMBER_OF_COMPUTE_RESOURCES_PER_QUEUE,
    MAX_NUMBER_OF_COMPUTE_RESOURCES,
    MAX_NUMBER_OF_COMPUTE_RESOURCES_PER_CLUSTER,
)
# from pcluster.constants import (
#     DEFAULT_MAX_COUNT,
#     DEFAULT_MIN_COUNT,
#     MAX_COMPUTE_RESOURCES_PER_QUEUE,
#     MAX_NUMBER_OF_COMPUTE_RESOURCES_PER_CLUSTER,
#     MAX_NUMBER_OF_QUEUES,
#     SUPPORTED_SCHEDULERS,
# )
from pcluster.utils import error, get_supported_os_for_scheduler
from pcluster.validators.cluster_validators import NameValidator
from pcluster.aws.aws_api import AWSApi
from pcluster.aws.common import get_region
from pcluster.cli.commands.configure.networking import (
    NetworkConfiguration,
    PublicPrivateNetworkConfig,
    automate_subnet_creation,
    automate_vpc_with_subnet_creation,
)
from questionary import Separator

from aws_pcluster_bootstrap_helpers.utils.logging import setup_logger

# TODO I think there is a bug in the constants
MAX_NUMBER_OF_COMPUTE_RESOURCES_PER_CLUSTER = 50
MAX_NUMBER_OF_QUEUES = 10
MAX_NUMBER_OF_COMPUTE_RESOURCES_PER_QUEUE = 5

logger = setup_logger("configure")
logger.propagate = False


def create_subnets():
    scheduler = "slurm"
    head_node_instance_type = "t3a.2xlarge"
    cluster_size = MAX_NUMBER_OF_COMPUTE_RESOURCES_PER_CLUSTER
    compute_instance_types = []
    network_configuration = PublicPrivateNetworkConfig()
    azs_for_head_node_type = AWSApi.instance().ec2.get_supported_az_for_instance_type(
        head_node_instance_type
    )
    azs_for_compute_types = _get_common_supported_az_for_multi_instance_types(
        compute_instance_types
    )
    common_availability_zones = set(azs_for_head_node_type) & set(azs_for_compute_types)
    vpc_parameters = _create_vpc_parameters(
        scheduler, head_node_instance_type, compute_instance_types, cluster_size
    )


def get_network_data(region: str = "us-east-1"):
    os.environ["AWS_DEFAULT_REGION"] = region
    network_data = _get_vpcs_and_subnets()
    return network_data



import datetime
from typing import Any, List, Optional
from pydantic import BaseModel
from jinja2 import Template
import json


class TerraformVariable(BaseModel):
    name: str
    default: Optional[Any] = None
    type: Optional[str] = None
    description: Optional[str] = None

    def render_to_terraform_variable(self):
        # Define the Jinja template
        d = self.__dict__
        template_string = """
        variable "{{item['name']}}" {% raw %} { {% endraw %}
            {% if item['type'] is not none %}
                type = "{{ item['type'] }}"
            {% endif %}
            {% if item['description'] is not none %}
                description = "{{ item['description'] }}"
            {% endif %}
            {% if item['default'] is not none %}
                default = "{{ item['default'] }}"
            {% endif %}
        {% raw %} } {% endraw %}
        """
        # Create the Jinja template object
        template = Template(template_string)

        # Render the template with the filtered data
        result = template.render(item=d)
        return result


class AWS(BaseModel):
    region: str = "us-east-1"
    vpc_id: str
    subnet_ids: List[str]


class BaseTags(BaseModel):
    name: str = "slurm"
    namespace: str = "bioanalyze"
    environment: str = "hpc"
    stage: str = "dev"


class PCluster(BaseModel):
    os: str = "ubuntu2004"
    build_ami: bool = False
    pcluster_ami_owner: str = "666813242062"
    pcluster_ami: str = "ami-0c7f8073ad4068592"
    pcluster_version: str = "3.5.0"
    head_node_instance_type: str = "t3a.medium"
    use_head_node_elastic_ip: bool = True


class Jhub(BaseModel):
    use_internal: bool = False
    domain: str
    acm: str


class EFS(BaseModel):
    mount_path: str
    use_existing_efs: bool = False
    efs_id: Optional[str] = None


class PClusterBootstrap(BaseModel):
    mounted_efs: List[EFS] = [EFS(mount_path="/apps"), EFS(mount_path="/scratch")]
    aws: AWS



import os
from copy import deepcopy
import boto3
from typing import Dict, Any, List, Optional
import pathlib
import time
import tempfile
import json
from pcluster.api.controllers.image_operations_controller import describe_image
from pcluster.api.models import DescribeImageResponseContent

from mypy_boto3_logs.client import CloudWatchLogsClient
from aws_pcluster_bootstrap_helpers.utils.logging import setup_logger
from aws_pcluster_bootstrap_helpers.utils.commands import run_bash_verbose

# todo change prefect to ansible
# from prefect import flow, task
# from prefect_shell import shell_run_command

from pcluster import utils

PCLUSTER_VERSION = utils.get_installed_version()

logger = setup_logger("build-ami")

BUILD_IN_PROGRESS = "BUILD_IN_PROGRESS"
BUILD_FAILED = "BUILD_FAILED"
BUILD_COMPLETE = "BUILD_COMPLETE"
DELETE_IN_PROGRESS = "DELETE_IN_PROGRESS"
DELETE_FAILED = "DELETE_FAILED"
DELETE_COMPLETE = "DELETE_COMPLETE"


def parse_image_status(data: DescribeImageResponseContent):
    image_status = data.image_build_status
    logger.info(f"Pcluster build status: {image_status}")
    if image_status == "BUILD_FAILED":
        raise Exception(f"Image build failed: {image_status}")
    elif "FAILED" in image_status:
        raise Exception(f"Image build failed: {image_status}")
    elif image_status == "BUILD_IN_PROGRESS":
        return True
    elif "PROGRESS" in image_status:
        return True
    elif "COMPLETE" not in image_status and "FAIL" not in image_status:
        return True
    elif "COMPLETE" in image_status:
        return False
    elif image_status == "BUILD_COMPLETE":
        return False
    else:
        raise Exception(f"Image status not compatible with bootstrap: {image_status}")


def build_ami_logs(
    data: DescribeImageResponseContent,
    log_client: CloudWatchLogsClient,
    start_time: int = 0,
):
    """

    'imageBuildLogsArn': 'arn:aws:logs:us-east-1:018835827632:log-group:/aws/imagebuilder/ParallelClusterImage-pcluster-3-2-1-wnl8bh-20221019'
    Parameters
    ----------
    data
    log_client
    start_time

    Returns
    -------

    """
    log_stream_name = data.image_build_logs_arn
    if not log_stream_name:
        return 0
    log_stream_name = log_stream_name.split(":").pop()
    log_group = deepcopy(log_stream_name)
    log_group = log_group.split("/")
    log_group.pop()
    log_group = list(filter(lambda x: x, log_group))
    log_group = "/".join(log_group)
    log_stream_name = log_stream_name.split("/").pop()
    start_time = print_logs(
        client=log_client,
        log_stream_name=log_stream_name,
        log_group=log_group,
        start_time=start_time,
    )
    return start_time


def build_in_progress(image_id: str, region="us-east-1"):
    os.environ["AWS_DEFAULT_REGION"] = region
    client = boto3.client("logs")
    start_time = 0

    build_in_process = True
    n = 1
    while build_in_process:
        logger.info(f"Pcluster: {image_id}, Region: {region}, N: {n}")
        build_data = describe_image(image_id=image_id, region=region)
        logger.info(build_data)
        try:
            start_time = build_ami_logs(
                data=build_data, log_client=client, start_time=start_time
            )
        except Exception as e:
            # logs only exist for a certain time
            # if they go away just skip this
            pass
        build_in_process = parse_image_status(build_data)
        print(
            f"Image Build:[{image_id} - {region}] Build in process: {build_in_process}"
        )

        n = n + 1
        # sleep for 10 minutes
        if build_in_process:
            time.sleep(600)
    return


def build_complete(image_id: str, output_file: str, region="us-east-1"):
    try:
        run_bash_verbose(
            command=f"""pcluster describe-image \\
      --image-id {image_id} \\
      -r {region} > {output_file} \\
    """,
            # return_all=True,
        )
    except Exception as e:
        return
    return


def start_build(image_id: str, region: str, config_file: str):
    try:
        run_bash_verbose(
            command=f"""pcluster build-image --image-id {image_id} -r {region} -c {config_file} """,
        )
    except Exception as e:
        return
    return


def build_ami_flow(
    image_id: str,
    output_file: pathlib.Path,
    config_file: pathlib.Path,
    region: str = "us-east-1",
    pcluster_version: str = "3.5.0",
):
    if pcluster_version not in PCLUSTER_VERSION:
        w = f"""Mismatch between specified pcluster version and installed
        Specified: {pcluster_version}, Installed: {PCLUSTER_VERSION}
        """
        raise ValueError(w)
    start_build(image_id, region, str(config_file))
    return True


def watch_ami_build_flow(
    image_id: str,
    output_file: pathlib.Path,
    config_file: pathlib.Path,
    region: str = "us-east-1",
) -> True:
    output_file = str(output_file)
    config_file = str(config_file)
    logger.info(f"Building ami: {image_id} {region}")
    start_build(
        image_id=image_id,
        region=region,
        config_file=config_file,
    )
    logger.info(f"Babysitting ami build: {image_id} {region}")
    build_in_progress(image_id=image_id, region=region)
    logger.info(f"Build complete!: {image_id} {region}")
    build_complete(image_id=image_id, region=region, output_file=output_file)
    logger.info(f"Build complete and data written to : {output_file}")
    return True


def watch_ami_flow(image_id: str, output_file: pathlib.Path, region: str = "us-east-1") -> bool:
    build_in_progress(image_id=image_id, region=region)
    build_complete(image_id=image_id, region=region, output_file=output_file)
    return True


def main(image_id: str, output_file: pathlib.Path, region: str = "us-east-1"):
    output_file = str(output_file)
    flow_state = watch_ami_build_flow(
        image_id=image_id, region=region, output_file=output_file
    )
    return flow_state



# Something is up with the pcluster constants
# from pcluster.constants import MAX_NUMBER_OF_QUEUES, MAX_NUMBER_OF_COMPUTE_RESOURCES_PER_CLUSTER
MAX_NUMBER_OF_QUEUES = 10
MAX_NUMBER_OF_COMPUTE_RESOURCES_PER_QUEUE = 5
MAX_NUMBER_OF_COMPUTE_RESOURCES = 50
MAX_NUMBER_OF_COMPUTE_RESOURCES_PER_CLUSTER = 50

PCLUSTER_AMIS = {

    "3.5.1": {
        "us-east-1": "",
        "us-east-2": "",
        "eu-west-1": "",
        "eu-west-2": "ami-031a47f98f4c6babd",
    }
}



import os
import numpy as np
from typing import List, Any, Dict, Optional
import questionary

from pcluster.aws.aws_api import AWSApi
import pandas as pd

from pcluster import utils
from datasize import DataSize
from questionary.prompts import common
from questionary.prompts.common import Choice, InquirerControl, Separator
from questionary.question import Question

from aws_pcluster_bootstrap_helpers.utils.logging import setup_logger

PCLUSTER_VERSION = utils.get_installed_version()
logger = setup_logger("configure-queues")

# from pcluster.constants import MAX_NUMBER_OF_QUEUES, MAX_NUMBER_OF_COMPUTE_RESOURCES_PER_CLUSTER
from aws_pcluster_bootstrap_helpers.utils.constants import (
    MAX_NUMBER_OF_QUEUES,
    MAX_NUMBER_OF_COMPUTE_RESOURCES_PER_QUEUE,
    MAX_NUMBER_OF_COMPUTE_RESOURCES,
    MAX_NUMBER_OF_COMPUTE_RESOURCES_PER_CLUSTER,
)

include_families = [
    "t3a",
    "m5a",
    "m5d",
    "m6i",
    "m6a",
    "c4",
    "c5",
    "c6",
    "r4",
    "r5",
    "r5a",
    "r5ad",
    "r5b",
    "r5d",
    "r5dn",
    "r5n",
    "r6a",
    "r6i",
    "r6id",
    # gpu types
    "g2",
    "g3",
    "g3s",
    "g4ad",
    "g4dn",
    "p2",
    "p3",
    "p3dn",
    "p4dn",
    "p4d",
]
exclude_families = ["m5dn"]
exclude_sizes = ["nano", "metal", "micro", "small", "medium"]
include_mems = [16, 32, 60, 128, 192, 356, 512, 768, 1024, 1536]


def size_in_gib(mib: int) -> int:
    mib_bytes = DataSize(f"{mib}Mi")
    return mib_bytes / mib_bytes.IEC_prefixes["Gi"]


def generate_preferred_families() -> List[Any]:
    return []


def get_instance_types(
    region: str = "us-east-1",
    architecture: str = "x86_64",
    include_families: Optional[List] = include_families,
    exclude_families: Optional[List] = exclude_families,
    include_gpus=True,
    exclude_sizes: Optional[List] = exclude_sizes,
    include_sizes: Optional[List] = None,
    mem_upper_limit: Optional = None,
    mem_lower_limit: Optional = None,
    vcpu_upper_limit: Optional = None,
    vcpu_lower_limit: Optional = None,
    include_mems: Optional[List[Any]] = include_mems,
    n_cpu_queues: int = 6,
    n_gpu_queues: int = 4,
):
    if n_gpu_queues + n_cpu_queues > MAX_NUMBER_OF_QUEUES:
        raise ValueError(
            f"Gpu queues + CPU Queues must be less than {MAX_NUMBER_OF_QUEUES}"
        )
    if include_families is None:
        include_families = []
    if exclude_families is None:
        exclude_families = []
    if exclude_sizes is None:
        exclude_sizes = []
    if include_sizes is None:
        include_sizes = []
    if include_mems is None:
        include_mems = []

    queues = {}
    logger.info(f"Getting instance types for region: {region}")
    os.environ["AWS_DEFAULT_REGION"] = region
    instance_types = AWSApi.instance().ec2.list_instance_types()
    instance_records = []
    instance_type_infos = []
    logger.info(
        f"Getting instance type info for {len(instance_types)} in region {region}"
    )
    for instance_type in instance_types:
        info = AWSApi.instance().ec2.get_instance_type_info(instance_type)
        instance_type = info.__dict__["instance_type_data"]
        family = instance_type["InstanceType"].split(".")[0]
        size = instance_type["InstanceType"].split(".")[1]

        size_in_mib = None
        bytes = None
        gibs = None
        arch = instance_type["ProcessorInfo"]["SupportedArchitectures"][0]
        if "MemoryInfo" in instance_type.keys():
            if "SizeInMib" in instance_type["MemoryInfo"].keys():
                size_in_mib = instance_type["MemoryInfo"]["SizeInMib"]
                bytes = DataSize(f"{size_in_mib}Mi")
                gibs = bytes / bytes.IEC_prefixes["Gi"]
                instance_type["MemoryInfo"]["SizeInGib"] = gibs
            elif "SizeInMiB" in instance_type["MemoryInfo"].keys():
                size_in_mib = instance_type["MemoryInfo"]["SizeInMiB"]
                bytes = DataSize(f"{size_in_mib}Mi")
                gibs = bytes / bytes.IEC_prefixes["Gi"]
                instance_type["MemoryInfo"]["SizeInGib"] = gibs
            else:
                raise ValueError(instance_type["MemoryInfo"])

        gpu = False
        if "GpuInfo" in instance_type:
            if "Gpus" in instance_type["GpuInfo"]:
                if len(instance_type["GpuInfo"]["Gpus"]):
                    gpu = True

        try:
            efa = instance_type["NetworkInfo"]["EfaSupported"]
        except Exception as e:
            efa = False
        instance_record = dict(
            family=family,
            arch=arch,
            size=size,
            efa=efa,
            instance_type=instance_type["InstanceType"],
            vcpus=instance_type["VCpuInfo"]["DefaultVCpus"],
            cores=instance_type["VCpuInfo"]["DefaultCores"],
            size_in_mib=size_in_mib,
            size_in_gibs=gibs,
            gpu=gpu,
        )
        instance_records.append(instance_record)

        instance_type_infos.append(instance_type)
    logger.info(f"Generating data frame")
    df = pd.DataFrame.from_records(instance_records)
    df = df[df["arch"].str.contains(architecture)]
    families = list(df["family"].unique())
    families_not_found = []
    if len(include_families):
        df = df[df["family"].str.contains("|".join(include_families))]
    if len(exclude_families):
        df = df[~df["family"].isin(exclude_families)]
    if len(exclude_sizes):
        df = df[~df["size"].isin(exclude_sizes)]

    # in us-east-1 this gets to about 48 instance types

    if not include_gpus:
        df = df[~df["gpu"]]

    if mem_upper_limit:
        df = df[df["size_in_gibs"] >= mem_upper_limit]
    if mem_lower_limit:
        df = df[df["size_in_gibs"] >= mem_lower_limit]

    if vcpu_upper_limit:
        df = df[df["vcpus"] >= vcpu_upper_limit]
    if vcpu_lower_limit:
        df = df[df["vcpus"] >= vcpu_lower_limit]

    df = df.sort_values(by=["size_in_gibs", "vcpus", "family"])
    cpu_df = df[~df["gpu"]]
    gpu_df = df[df["gpu"]]

    n_gpu_instance_types = n_gpu_queues * MAX_NUMBER_OF_COMPUTE_RESOURCES
    n_cpu_instance_types = n_cpu_queues * MAX_NUMBER_OF_COMPUTE_RESOURCES
    if gpu_df.shape[0] <= n_gpu_instance_types:
        # include all the gpus
        gpu_dfs = np.array_split(gpu_df, MAX_NUMBER_OF_COMPUTE_RESOURCES)
        for i, t_gpu_df in enumerate(gpu_dfs):
            n = i + 1
            queues[f"gpu-{n}"] = t_gpu_df.to_dict("records")

    if len(include_mems):
        cpu_df = cpu_df[cpu_df["size_in_gibs"].isin(include_mems)]

    logger.info("Complete get instance types")
    return dict(gpu_df=gpu_df, cpu_df=cpu_df, all_df=df)


def write_instance_types_csvs(gpu_df: pd.DataFrame, cpu_df: pd.DataFrame):
    logger.info("Writing out gpu and cpu instance types...")
    gpu_df.to_csv("pcluster_gpu_instances.csv", index=False)
    cpu_df.to_csv("pcluster_cpu_instances.csv", index=False)


def prompt_for_instance_types(region="us-east-1"):
    dfs = get_instance_types(region=region)
    cpu_df = dfs["cpu_df"]
    gpu_df = dfs["gpu_df"]
    include_gpus = questionary.confirm("Create queues from GPU Instances?").ask()
    if include_gpus:
        cpu_queues = questionary.select(
            "How many cpu queues?", choices=list(range(1, MAX_NUMBER_OF_QUEUES))
        ).ask()
        gpu_queues = MAX_NUMBER_OF_QUEUES - cpu_queues
    else:
        cpu_queues = 10
        gpu_queues = 0

    n_cpu_instance_types = cpu_queues * MAX_NUMBER_OF_COMPUTE_RESOURCES
    n_gpu_instance_types = gpu_queues * MAX_NUMBER_OF_COMPUTE_RESOURCES

    cpu_queues_data = []
    cpu_labels = (
        cpu_df["instance_type"]
        + " | "
        + cpu_df["size_in_gibs"].map(str)
        + " gib"
        + " | "
        + cpu_df["vcpus"].map(str)
        + " cores"
    )
    cpu_labels = cpu_labels.map(str)
    cpu_labels = list(cpu_labels)

    gpu_labels = (
        gpu_df["instance_type"]
        + " | "
        + gpu_df["size_in_gibs"].map(str)
        + " gib"
        + " | "
        + gpu_df["vcpus"].map(str)
        + " cores"
    )
    gpu_labels = gpu_labels.map(str)
    gpu_labels = list(gpu_labels)

    cpu_min_mems = cpu_df["size_in_gibs"].min()
    cpu_max_mems = cpu_df["size_in_gibs"].max()
    gpu_min_mems = gpu_df["size_in_gibs"].min()
    gpu_max_mems = gpu_df["size_in_gibs"].max()
    cpu_mems = cpu_df["size_in_gibs"].map(str).unique().tolist()
    gpu_mems = gpu_df["size_in_gibs"].unique().tolist()
    cpu_families = cpu_df["family"].unique().tolist()
    gpu_families = gpu_df["family"].unique().tolist()

    logger.info("###################################################")
    logger.info(f"Begin selection for CPU Queues")
    logger.info(f"Choose a maximum of : {n_cpu_instance_types}")
    logger.info("###################################################")
    cpu_min_mem = questionary.select(
        "CPU min memory", choices=cpu_mems, default=cpu_mems[0]
    ).ask()
    cpu_max_mem = questionary.select(
        "CPU max memory", choices=cpu_mems, default=cpu_mems[-1]
    ).ask()
    t_cpu_df = cpu_df[cpu_df["size_in_gibs"] >= float(cpu_min_mem)]
    t_cpu_df = t_cpu_df[t_cpu_df["size_in_gibs"] <= float(cpu_max_mem)]
    families = t_cpu_df["family"].unique().tolist()
    cpu_families = questionary.checkbox(
        "CPU families",
        choices=list(map(lambda x: Choice(x, checked=True), families)),
    ).ask()
    t_cpu_df = t_cpu_df[t_cpu_df["family"].isin(cpu_families)]

    cpu_labels = (
        t_cpu_df["instance_type"]
        + " | "
        + t_cpu_df["size_in_gibs"].map(str)
        + " gib"
        + " | "
        + t_cpu_df["vcpus"].map(str)
        + " cores"
    )
    cpu_labels = cpu_labels.map(str)
    cpu_labels = list(cpu_labels)

    instance_types = questionary.checkbox(
        f"Select instance types: max {n_cpu_instance_types}",
        choices=cpu_labels,
        validate=lambda x: True
        if len(x) > n_cpu_instance_types
        else f"Please choose a maximum of: {n_cpu_instance_types}",
    ).ask()

    queue_rules = [
        # dev
        dict(
            min_mem=16,
            max_mem=96,
            preferred_families=[
                "m5",
                "m5a",
                "m5ad",
                "m5d",
                "m5dn",
                "m5n",
                "m5zn",
                "m6a",
                "m6i",
                "m6id",
                "m5",
                "m5a",
                "m6a",
                "t3a",
            ],
        ),
        # compute optimized
        dict(
            min_mem=96,
            max_mem=488,
            preferred_families=[
                "c1",
                "c3",
                "c4",
                "c5",
                "c5a",
                "c5ad",
                "c5d",
                "c5n",
                "c6a",
                "c6i",
                "c6id",
                "cc2",
                "c5",
                "c5a",
                "c4",
            ],
        ),
        # mem optimized
        dict(
            min_mem=96,
            max_mem=488,
            preferred_families=[
                "r3",
                "r4",
                "r5",
                "r5a",
                "r5ad",
                "r5b",
                "r5d",
                "r5dn",
                "r5n",
                "r6a",
                "r6i",
                "r6id",
            ],
        ),
        dict(
            min_mem=96,
            max_mem=488,
            prefrred_families=[
                "r3",
                "r4",
                "r5",
                "r5a",
                "r5ad",
                "r5b",
                "r5d",
                "r5dn",
                "r5n",
                "r6a",
                "r6i",
                "r6id",
                "c1",
                "c3",
                "c4",
                "c5",
                "c5a",
                "c5ad",
                "c5d",
                "c5n",
                "c6a",
                "c6i",
                "c6id",
                "cc2",
                "c5",
                "c5a",
                "c4",
            ],
        ),
        dict(min_mem=488, max_mem=768),
        dict(min_mem=488, max_mem=None),
    ]
    queues = {}
    for i, queue_rule in enumerate(queue_rules):
        if queue_rule["min_mem"]:
            cpu_queue = cpu_df[cpu_df["size_in_gibs"] >= 16]
        else:
            cpu_queue = cpu_df
        if queue_rule["max_mem"]:
            cpu_queue = cpu_queue[cpu_queue["size_in_gibs"] <= 96]

        if cpu_queue.shape[0] > MAX_NUMBER_OF_COMPUTE_RESOURCES:
            if queue_rule.get("preferred_families", None):
                cpu_queue = cpu_queue[
                    cpu_queue["family"].isin(queue_rule["preferred_families"])
                ]
            else:
                cpu_queue = cpu_queue[cpu_queue["family"].isin(include_families)]

        queues[f"cpu-{i + 1}"] = cpu_queue
        mems = cpu_queue["size_in_gibs"].unique().tolist()
        min_mem = cpu_queue["size_in_gibs"].min()
        max_mem = cpu_queue["size_in_gibs"].max()
        bins = pd.cut(
            cpu_queue["size_in_gibs"].unique(), bins=MAX_NUMBER_OF_COMPUTE_RESOURCES
        )



"""Console script for aws_pcluster_bootstrap_helpers."""
import os

import json
import pathlib
import typer
from aws_pcluster_bootstrap_helpers.commands import (
    cli_build_ami,
    cli_instance_types,
    cli_create_cluster,
    cli_configure,
)
from aws_pcluster_bootstrap_helpers.utils.logging import setup_logger

logger = setup_logger("cli")

cli = typer.Typer()


def region_ctx(region):
    os.environ["AWS_DEFAULT_REGION"] = region


@cli.command("watch-ami")
def watch_ami_build_cli(
    output: pathlib.Path = typer.Option(
        ...,
        help="Path to output of pcluster describe-image",
    ),
    region: str = typer.Option(
        os.environ.get("AWS_DEFAULT_REGION", "us-east-1"),
    ),
    image_id: str = typer.Option(..., help="""Image ID for build"""),
):
    """
    Watcher for PCluster image builder.
    Given an image_id and region it will wait for the image to build and report back.
    """
    cli_build_ami.watch_ami_build_flow(
        image_id=image_id, output_file=output, region=region
    )


@cli.command("build-and-watch-ami")
def build_and_watch_ami_cli(
    pcluster_version: str = typer.Option(
        "3.2", help="PCluster version used to build the image"
    ),
    output: pathlib.Path = typer.Option(
        ...,
        help="Path to output of pcluster describe-image",
    ),
    config_file: pathlib.Path = typer.Option(
        ..., help="Path to build config for the pcluster ami"
    ),
    region: str = typer.Option(
        os.environ.get("AWS_DEFAULT_REGION", "us-east-1"),
    ),
    image_id: str = typer.Option(..., help="""Image ID for build"""),
):
    """
    Start to build a pcluster AMI and wait for the build to complete
    """
    cli_build_ami.watch_ami_build_flow(
        image_id=image_id,
        config_file=config_file,
        output_file=output,
        region=region,
    )


@cli.command("create-and-watch-cluster")
def create_and_watch_cluster(
    pcluster_version: str = typer.Option(
        "3.2", help="PCluster version used to build the image"
    ),
    config_file: pathlib.Path = typer.Option(
        ..., help="Path to pcluster configure file."
    ),
    region: str = typer.Option(
        os.environ.get("AWS_DEFAULT_REGION", "us-east-1"),
    ),
    cluster_name: str = typer.Option(..., help="""Cluster name"""),
):
    """
    Run:
        `pcluster create-cluster -n cluster_name -r region`.
    Then wait for the cluster to complete.
    """
    cli_create_cluster.watch_cluster_create_flow(
        region=region,
        config_file=config_file,
        cluster_name=cluster_name,
    )
    return


@cli.command("instance-types")
def get_instance_types(
    region: str = typer.Option(
        os.environ.get("AWS_DEFAULT_REGION", "us-east-1"),
    ),
):
    """
    Get all available instance types for your region and write them to a dataframe
    """
    dfs = cli_instance_types.get_instance_types(region=region)
    cli_instance_types.write_instance_types_csvs(
        gpu_df=dfs["gpu_df"], cpu_df=dfs["cpu_df"]
    )


@cli.command("configure")
def configure(
    region: str = typer.Option("us-east-1", prompt=True),
    name: str = typer.Option("slurm", prompt=False),
    namespace: str = typer.Option(
        ..., prompt=True, help="Typically the company name. Example: my_company"
    ),
    environment: str = typer.Option(
        "hpc", prompt=False, help="Type of deployment. HPC, Kubernetes, etc."
    ),
    stage: str = typer.Option(
        "dev", prompt=False, help="Development env. Example: dev, test, stage, prod."
    ),
    force: bool = typer.Option(True, help="Force overwrite of existing config file."),
):
    """
    Generate the json file used for the rest of the bootstrap
    """
    id = f"{namespace}-{environment}-{stage}-{name}"
    json_file = f"{id}.json"
    os.environ["AWS_DEFAULT_REGION"] = region
    data = cli_configure.configure(region=region)
    configure_data = {
        "namespace": namespace,
        "environment": environment,
        "stage": stage,
        "name": name,
        "region": region,
        "terraform_vars": data['network_data'],
    }
    configure_data.update(data['dns'])
    configure_data.update(data['efs'])
    configure_data.update(data['elastic_ip'])
    logger.info(f"Writing file to : {json_file}")
    logger.info(configure_data)
    with open(json_file, "w") as fh:
        json.dump(configure_data, fh)
    return


if __name__ == "__main__":
    cli()



import os
import boto3
import questionary

import boto3
from typing import List, Any, Dict, Optional


def get_efs(region: str = "us-east-1") -> List[Dict[str, Any]]:
    efs_client = boto3.client("efs", region_name=region)

    response = efs_client.describe_file_systems()

    file_systems = response["FileSystems"]
    while response.get("NextMarker"):
        response = efs_client.describe_file_systems(Marker=response["NextMarker"])
        file_systems.extend(response["FileSystems"])

    for file_system in file_systems:
        file_system['label'] = f"{file_system['Name']} - {file_system['FileSystemId']}"

    return file_systems
